@@ -46,16 +46,26 @@ static const char *creg_prefix[] = { "+CREG:", NULL };
 static const char *cops_prefix[] = { "+COPS:", NULL };
 static const char *csq_prefix[] = { "+CSQ:", NULL };
 static const char *cind_prefix[] = { "+CIND:", NULL };
-
-#define SIGNAL_STRENGTH_IND 2
+static const char *option_tech_prefix[] = { "_OCTI:", "_OUWCTI:", NULL };
 
 struct netreg_data {
 	GAtChat *chat;
 	char mcc[OFONO_MAX_MCC_LENGTH + 1];
 	char mnc[OFONO_MAX_MNC_LENGTH + 1];
+	int signal_index; /* If strength is reported via CIND */
+	int signal_min; /* min strength reported via CIND */
+	int signal_max; /* max strength reported via CIND */
+	int tech;
 	unsigned int vendor;
 };
 
+struct tech_query {
+	int status;
+	int lac;
+	int ci;
+	struct ofono_netreg *netreg;
+};
+
 static void extract_mcc_mnc(const char *str, char *mcc, char *mnc)
 {
 	/* Three digit country code */
@@ -67,6 +77,64 @@ static void extract_mcc_mnc(const char *str, char *mcc, char *mnc)
 	mnc[OFONO_MAX_MNC_LENGTH] = '\0';
 }
 
+static int option_parse_tech(GAtResult *result)
+{
+	GAtResultIter iter;
+	int s, octi, ouwcti;
+	int tech = -1;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "_OCTI:"))
+		return -1;
+
+	if (!g_at_result_iter_next_number(&iter, &s))
+		return -1;
+
+	if (!g_at_result_iter_next_number(&iter, &octi))
+		return -1;
+
+	if (!g_at_result_iter_next(&iter, "_OUWCTI:"))
+		return -1;
+
+	if (!g_at_result_iter_next_number(&iter, &s))
+		return -1;
+
+	if (!g_at_result_iter_next_number(&iter, &ouwcti))
+		return -1;
+
+	switch (octi) {
+	case 1: /* GSM */
+		tech = 0;
+		break;
+	case 2: /* GPRS */
+		tech = 1;
+		break;
+	case 3: /* EDGE */
+		tech = 3;
+		break;
+	}
+
+	switch (ouwcti) {
+	case 1: /* UMTS */
+		tech = 2;
+		break;
+	case 2: /* HSDPA */
+		tech = 4;
+		break;
+	case 3: /* HSUPA */
+		tech = 5;
+		break;
+	case 4: /* HSPA */
+		tech = 6;
+		break;
+	}
+
+	DBG("octi %d ouwcti %d tech %d", octi, ouwcti, tech);
+
+	return tech;
+}
+
 static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -75,7 +143,6 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_error error;
 	struct netreg_data *nd = cbd->user;
 
-	dump_response("at_creg_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -89,9 +156,24 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
+	if ((status == 1 || status == 5) && (tech == -1))
+		tech = nd->tech;
+
 	cb(&error, status, lac, ci, tech, cbd->data);
 }
 
+static void option_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct ofono_netreg *netreg = cbd->data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+
+	if (ok)
+		nd->tech = option_parse_tech(result);
+	else
+		nd->tech = -1;
+}
+
 static void at_registration_status(struct ofono_netreg *netreg,
 					ofono_netreg_status_cb_t cb,
 					void *data)
@@ -104,6 +186,35 @@ static void at_registration_status(struct ofono_netreg *netreg,
 
 	cbd->user = nd;
 
+	switch (nd->vendor) {
+	case OFONO_VENDOR_MBM:
+		/*
+		 * Send *ERINFO to find out the current tech, it will be
+		 * intercepted in mbm_erinfo_notify
+		 */
+		g_at_chat_send(nd->chat, "AT*ERINFO?", none_prefix,
+				NULL, NULL, NULL);
+		break;
+	case OFONO_VENDOR_NOVATEL:
+		/*
+		 * Send $CNTI=0 to find out the current tech, it will be
+		 * intercepted in nw_cnti_notify
+		 */
+		g_at_chat_send(nd->chat, "AT$CNTI=0", none_prefix,
+				NULL, NULL, NULL);
+		break;
+	case OFONO_VENDOR_OPTION_HSO:
+		/*
+		 * Send AT_OCTI?;_OUWCTI? to find out the current tech,
+		 * option_tech_cb will call fire CREG? to do the rest.
+		 */
+		if (g_at_chat_send(nd->chat, "AT_OCTI?;_OUWCTI?",
+					option_tech_prefix,
+					option_tech_cb, cbd, NULL) == 0)
+			nd->tech = -1;
+		break;
+	}
+
 	if (g_at_chat_send(nd->chat, "AT+CREG?", creg_prefix,
 				at_creg_cb, cbd, g_free) > 0)
 		return;
@@ -126,13 +237,10 @@ static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	const char *name;
 	struct ofono_error error;
 
-	dump_response("cops_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
-	if (!ok || nd->mcc[0] == '\0' || nd->mnc[0] == '\0') {
-		cb(&error, NULL, cbd->data);
-		goto out;
-	}
+	if (!ok)
+		goto error;
 
 	g_at_result_iter_init(&iter, result);
 
@@ -166,17 +274,15 @@ static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	op.status = 2;
 	op.tech = tech;
 
-	ofono_debug("cops_cb: %s, %s %s %d", name, nd->mcc, nd->mnc, tech);
+	DBG("cops_cb: %s, %s %s %d", name, nd->mcc, nd->mnc, tech);
 
 	cb(&error, &op, cbd->data);
-
-out:
 	g_free(cbd);
 
 	return;
 
 error:
-	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+	cb(&error, NULL, cbd->data);
 
 	g_free(cbd);
 }
@@ -185,11 +291,14 @@ static void cops_numeric_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	struct netreg_data *nd = ofono_netreg_get_data(cbd->user);
+	ofono_netreg_operator_cb_t cb = cbd->cb;
 	GAtResultIter iter;
 	const char *str;
 	int format;
+	int len;
+	struct ofono_error error;
 
-	dump_response("cops_numeric_cb", ok, result);
+	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok)
 		goto error;
@@ -206,19 +315,31 @@ static void cops_numeric_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (ok == FALSE || format != 2)
 		goto error;
 
-	if (g_at_result_iter_next_string(&iter, &str) == FALSE ||
-		strlen(str) == 0)
+	if (g_at_result_iter_next_string(&iter, &str) == FALSE)
+		goto error;
+
+	len = strspn(str, "0123456789");
+
+	if (len != 5 && len != 6)
 		goto error;
 
 	extract_mcc_mnc(str, nd->mcc, nd->mnc);
 
-	ofono_debug("Cops numeric got mcc: %s, mnc: %s", nd->mcc, nd->mnc);
+	DBG("Cops numeric got mcc: %s, mnc: %s", nd->mcc, nd->mnc);
 
-	return;
+	ok = g_at_chat_send(nd->chat, "AT+COPS=3,0", none_prefix,
+					NULL, NULL, NULL);
+
+	if (ok)
+		ok = g_at_chat_send(nd->chat, "AT+COPS?", cops_prefix,
+					cops_cb, cbd, NULL);
+
+	if (ok)
+		return;
 
 error:
-	nd->mcc[0] = '\0';
-	nd->mnc[0] = '\0';
+	cb(&error, NULL, cbd->data);
+	g_free(cbd);
 }
 
 static void at_current_operator(struct ofono_netreg *netreg,
@@ -240,14 +361,6 @@ static void at_current_operator(struct ofono_netreg *netreg,
 		ok = g_at_chat_send(nd->chat, "AT+COPS?", cops_prefix,
 					cops_numeric_cb, cbd, NULL);
 
-	if (ok)
-		ok = g_at_chat_send(nd->chat, "AT+COPS=3,0", none_prefix,
-					NULL, NULL, NULL);
-
-	if (ok)
-		ok = g_at_chat_send(nd->chat, "AT+COPS?", cops_prefix,
-					cops_cb, cbd, NULL);
-
 	if (ok)
 		return;
 
@@ -267,7 +380,6 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int num = 0;
 	struct ofono_error error;
 
-	dump_response("cops_list_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -282,7 +394,7 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			num += 1;
 	}
 
-	ofono_debug("Got %d elements", num);
+	DBG("Got %d elements", num);
 
 	list = g_try_new0(struct ofono_network_operator, num);
 
@@ -343,15 +455,15 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		}
 	}
 
-	ofono_debug("Got %d operators", num);
+	DBG("Got %d operators", num);
 
 {
 	int i = 0;
 
 	for (; i < num; i++) {
-		ofono_debug("Operator: %s, %s, %s, status: %d, %d",
-				list[i].name, list[i].mcc, list[i].mnc,
-				list[i].status, list[i].tech);
+		DBG("Operator: %s, %s, %s, status: %d, %d",
+			list[i].name, list[i].mcc, list[i].mnc,
+			list[i].status, list[i].tech);
 	}
 }
 
@@ -386,7 +498,6 @@ static void register_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_netreg_register_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("register_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	cb(&error, cbd->data);
@@ -423,7 +534,7 @@ static void at_register_manual(struct ofono_netreg *netreg,
 	if (!cbd)
 		goto error;
 
-	sprintf(buf, "AT+COPS=1,2,\"%s%s\"", mcc, mnc);
+	snprintf(buf, sizeof(buf), "AT+COPS=1,2,\"%s%s\"", mcc, mnc);
 
 	if (g_at_chat_send(nd->chat, buf, none_prefix,
 				register_cb, cbd, g_free) > 0)
@@ -456,27 +567,12 @@ error:
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
-static inline void report_signal_strength(struct ofono_netreg *netreg,
-						int strength)
-{
-	ofono_debug("csq_notify: %d", strength);
-
-	if (strength == 99)
-		strength = -1;
-	else
-		strength = (strength * 100) / 31;
-
-	ofono_netreg_strength_notify(netreg, strength);
-}
-
 static void csq_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	int strength;
 	GAtResultIter iter;
 
-	dump_response("csq_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CSQ:"))
@@ -485,7 +581,8 @@ static void csq_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	report_signal_strength(netreg, strength);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
 static void calypso_csq_notify(GAtResult *result, gpointer user_data)
@@ -494,8 +591,6 @@ static void calypso_csq_notify(GAtResult *result, gpointer user_data)
 	int strength;
 	GAtResultIter iter;
 
-	dump_response("calypso_csq_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "%CSQ:"))
@@ -504,7 +599,8 @@ static void calypso_csq_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	report_signal_strength(netreg, strength);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
 static void option_osigq_notify(GAtResult *result, gpointer user_data)
@@ -513,8 +609,6 @@ static void option_osigq_notify(GAtResult *result, gpointer user_data)
 	int strength;
 	GAtResultIter iter;
 
-	dump_response("option_osigq_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "_OSIGQ:"))
@@ -523,49 +617,17 @@ static void option_osigq_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	report_signal_strength(netreg, strength);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
-static void option_owcti_notify(GAtResult *result, gpointer user_data)
-{
-	int mode;
-	GAtResultIter iter;
-
-	g_at_result_iter_init(&iter, result);
-
-	if (!g_at_result_iter_next(&iter, "_OWCTI:"))
-		return;
-
-	if (!g_at_result_iter_next_number(&iter, &mode))
-		return;
-
-	ofono_info("OWCTI mode: %d", mode);
-}
-
-static void option_octi_notify(GAtResult *result, gpointer user_data)
-{
-	int mode;
-	GAtResultIter iter;
-
-	g_at_result_iter_init(&iter, result);
-
-	if (!g_at_result_iter_next(&iter, "_OCTI:"))
-		return;
-
-	if (!g_at_result_iter_next_number(&iter, &mode))
-		return;
-
-	ofono_info("OCTI mode: %d", mode);
-}
-
-static void ste_ciev_notify(GAtResult *result, gpointer user_data)
+static void ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
 	int strength, ind;
 	GAtResultIter iter;
 
-	dump_response("ciev_notify", TRUE, result);
-
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CIEV:"))
@@ -574,24 +636,26 @@ static void ste_ciev_notify(GAtResult *result, gpointer user_data)
 	if (!g_at_result_iter_next_number(&iter, &ind))
 		return;
 
-	if (ind == SIGNAL_STRENGTH_IND) {
-		if (!g_at_result_iter_next_number(&iter, &strength))
-			return;
+	if (ind != nd->signal_index)
+		return;
 
-		strength = (strength * 100) / 5;
-		ofono_netreg_strength_notify(netreg, strength);
-	}
+	if (!g_at_result_iter_next_number(&iter, &strength))
+		return;
+
+	strength = (strength * 100) / (nd->signal_max - nd->signal_min);
+	ofono_netreg_strength_notify(netreg, strength);
 }
 
-static void ste_cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
+static void cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_netreg_strength_cb_t cb = cbd->cb;
+	struct netreg_data *nd = cbd->user;
+	int index;
 	int strength;
 	GAtResultIter iter;
 	struct ofono_error error;
 
-	dump_response("cind_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -606,30 +670,32 @@ static void ste_cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	/* Skip battery charge level, which is the first reported  */
-	g_at_result_iter_skip_next(&iter);
+	for (index = 1; index < nd->signal_index; index++)
+		g_at_result_iter_skip_next(&iter);
 
 	g_at_result_iter_next_number(&iter, &strength);
 
-	strength = (strength * 100) / 5;
+	strength = (strength * 100) / (nd->signal_max - nd->signal_min);
 
 	cb(&error, strength, cbd->data);
 }
 
-static void option_ossysi_notify(GAtResult *result, gpointer user_data)
+static void huawei_rssi_notify(GAtResult *result, gpointer user_data)
 {
-	int mode;
+	struct ofono_netreg *netreg = user_data;
 	GAtResultIter iter;
+	int strength;
 
 	g_at_result_iter_init(&iter, result);
 
-	if (!g_at_result_iter_next(&iter, "_OSSYSI:"))
+	if (!g_at_result_iter_next(&iter, "^RSSI:"))
 		return;
 
-	if (!g_at_result_iter_next_number(&iter, &mode))
+	if (!g_at_result_iter_next_number(&iter, &strength))
 		return;
 
-	ofono_info("OSSYSI mode: %d", mode);
+	ofono_netreg_strength_notify(netreg,
+				at_util_convert_signal_strength(strength));
 }
 
 static void csq_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -640,7 +706,6 @@ static void csq_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	GAtResultIter iter;
 	struct ofono_error error;
 
-	dump_response("csq_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok) {
@@ -657,7 +722,7 @@ static void csq_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_result_iter_next_number(&iter, &strength);
 
-	ofono_debug("csq_cb: %d", strength);
+	DBG("csq_cb: %d", strength);
 
 	if (strength == 99)
 		strength = -1;
@@ -676,9 +741,15 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 	if (!cbd)
 		goto error;
 
-	if (nd->vendor == OFONO_VENDOR_STE) {
+	cbd->user = nd;
+
+	/*
+	 * If we defaulted to using CIND, then keep using it,
+	 * otherwise fall back to CSQ
+	 */
+	if (nd->signal_index > 0) {
 		if (g_at_chat_send(nd->chat, "AT+CIND?", cind_prefix,
-				ste_cind_cb, cbd, g_free) > 0)
+					cind_cb, cbd, g_free) > 0)
 			return;
 	} else {
 		if (g_at_chat_send(nd->chat, "AT+CSQ", csq_prefix,
@@ -693,23 +764,205 @@ error:
 	CALLBACK_WITH_FAILURE(cb, -1, data);
 }
 
+static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	int mode, gsm, umts;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "*ERINFO:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &mode) == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &gsm) == FALSE)
+		return;
+
+	/*
+	 * According to MBM the ERINFO unsolicited response does not contain
+	 * the mode parameter, however at least the MD300 does report it.  So
+	 * we handle both 2 and 3 argument versions
+	 */
+	if (g_at_result_iter_next_number(&iter, &umts) == FALSE) {
+		gsm = mode;
+		umts = gsm;
+	}
+
+	ofono_info("network capability: GSM %d UMTS %d", gsm, umts);
+
+	/* Convert to tech values from 27.007 */
+	switch (gsm) {
+	case 1: /* GSM */
+		nd->tech = 0;
+		break;
+	case 2: /* EDGE */
+		nd->tech = 3;
+		break;
+	default:
+		nd->tech = -1;
+	}
+
+	switch (umts) {
+	case 1: /* UMTS */
+		nd->tech = 2;
+		break;
+	case 2: /* UMTS + HSDPA */
+		nd->tech = 4;
+		break;
+	default:
+		break;
+	}
+}
+
+static void nw_cnti_notify(GAtResult *result, gpointer user_data)
+{
+	//struct ofono_netreg *netreg = user_data;
+	//struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	const char *tech;
+	int option;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (g_at_result_iter_next(&iter, "$CNTI:") == FALSE)
+		return;
+
+	if (g_at_result_iter_next_number(&iter, &option) == FALSE)
+		return;
+
+	if (option != 0)
+		return;
+
+	if (g_at_result_iter_next_unquoted_string(&iter, &tech) == FALSE)
+		return;
+
+	ofono_info("CNTI: %s", tech);
+}
+
+static void option_query_tech_cb(gboolean ok,
+			GAtResult *result, gpointer user_data)
+{
+	struct tech_query *tq = user_data;
+	int tech = -1;
+
+	if (ok)
+		tech = option_parse_tech(result);
+
+	ofono_netreg_status_notify(tq->netreg,
+			tq->status, tq->lac, tq->ci, tech);
+}
+
 static void creg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	int status, lac, ci, tech;
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
-
-	dump_response("creg_notify", TRUE, result);
+	struct tech_query *tq;
 
 	if (at_util_parse_reg_unsolicited(result, "+CREG:", &status,
 				&lac, &ci, &tech, nd->vendor) == FALSE)
 		return;
 
+	if (status != 1 && status != 5)
+		goto notify;
+
+	switch (nd->vendor) {
+	case OFONO_VENDOR_OPTION_HSO:
+		tq = g_new0(struct tech_query, 1);
+		if (!tq)
+			break;
+
+		tq->status = status;
+		tq->lac = lac;
+		tq->ci = ci;
+		tq->netreg = netreg;
+
+		if (g_at_chat_send(nd->chat, "AT_OCTI?;_OUWCTI?",
+					option_tech_prefix,
+					option_query_tech_cb, tq, g_free) > 0)
+			return;
+
+		g_free(tq);
+		break;
+	}
+
+	if ((status == 1 || status == 5) && tech == -1)
+		tech = nd->tech;
+
+notify:
 	ofono_netreg_status_notify(netreg, status, lac, ci, tech);
 }
 
-static void at_network_registration_initialized(gboolean ok, GAtResult *result,
-							gpointer user_data)
+static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_netreg *netreg = user_data;
+	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	GAtResultIter iter;
+	const char *str;
+	int index;
+	int min, max;
+
+	if (!ok)
+		goto error;
+
+	g_at_result_iter_init(&iter, result);
+	if (!g_at_result_iter_next(&iter, "+CIND:"))
+		goto error;
+
+	index = 1;
+
+	while (g_at_result_iter_open_list(&iter)) {
+		if (!g_at_result_iter_next_string(&iter, &str))
+			goto error;
+
+		if (!g_at_result_iter_open_list(&iter))
+			goto error;
+
+		while (g_at_result_iter_next_range(&iter, &min, &max))
+			;
+
+		if (!g_at_result_iter_close_list(&iter))
+			goto error;
+
+		if (!g_at_result_iter_close_list(&iter))
+			goto error;
+
+		if (g_str_equal("signal", str) == TRUE) {
+			nd->signal_index = index;
+			nd->signal_min = min;
+			nd->signal_max = max;
+		}
+
+		index += 1;
+	}
+
+	if (nd->signal_index == 0)
+		goto error;
+
+	g_at_chat_send(nd->chat, "AT+CMER=3,0,0,1", NULL,
+			NULL, NULL, NULL);
+	g_at_chat_register(nd->chat, "+CIEV:",
+				ciev_notify, FALSE, netreg, NULL);
+	g_at_chat_register(nd->chat, "+CREG:",
+				creg_notify, FALSE, netreg, NULL);
+
+	ofono_netreg_register(netreg);
+	return;
+
+error:
+	ofono_error("This driver is not setup with Signal Strength reporting"
+			" via CIND indications, please write proper netreg"
+			" handling for this device");
+
+	ofono_netreg_remove(netreg);
+}
+
+
+static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	struct netreg_data *nd = ofono_netreg_get_data(netreg);
@@ -720,68 +973,59 @@ static void at_network_registration_initialized(gboolean ok, GAtResult *result,
 		return;
 	}
 
-	g_at_chat_register(nd->chat, "+CREG:",
-				creg_notify, FALSE, netreg, NULL);
-	g_at_chat_register(nd->chat, "+CSQ:",
-				csq_notify, FALSE, netreg, NULL);
-
 	switch (nd->vendor) {
+	case OFONO_VENDOR_PHONESIM:
+		g_at_chat_register(nd->chat, "+CSQ:",
+					csq_notify, FALSE, netreg, NULL);
+		break;
 	case OFONO_VENDOR_CALYPSO:
 		g_at_chat_send(nd->chat, "AT%CSQ=1", none_prefix,
 				NULL, NULL, NULL);
 		g_at_chat_register(nd->chat, "%CSQ:", calypso_csq_notify,
 					FALSE, netreg, NULL);
-
 		break;
 	case OFONO_VENDOR_OPTION_HSO:
 		g_at_chat_send(nd->chat, "AT_OSSYS=1", none_prefix,
 				NULL, NULL, NULL);
-		g_at_chat_send(nd->chat, "AT_OCTI=1", none_prefix,
-				NULL, NULL, NULL);
 		g_at_chat_send(nd->chat, "AT_OSQI=1", none_prefix,
 				NULL, NULL, NULL);
 		g_at_chat_register(nd->chat, "_OSIGQ:", option_osigq_notify,
 					FALSE, netreg, NULL);
-		g_at_chat_register(nd->chat, "_OWCTI:", option_owcti_notify,
-					FALSE, netreg, NULL);
-		g_at_chat_register(nd->chat, "_OCTI:", option_octi_notify,
-					FALSE, netreg, NULL);
-		g_at_chat_register(nd->chat, "_OSSYSI:", option_ossysi_notify,
-					FALSE, netreg, NULL);
 
 		g_at_chat_send(nd->chat, "AT_OSSYS?", none_prefix,
 				NULL, NULL, NULL);
-		g_at_chat_send(nd->chat, "AT_OCTI?", none_prefix,
-				NULL, NULL, NULL);
 		g_at_chat_send(nd->chat, "AT_OSQI?", none_prefix,
 				NULL, NULL, NULL);
-
-		/*
-		 * Option has the concept of Speech Service versus
-		 * Data Service. Problem is that in Data Service mode
-		 * the card will reject all voice calls. This is a
-		 * problem for Multi-SIM cards where one of the SIM
-		 * cards is used in a mobile phone and thus incoming
-		 * calls would be not signalled on the phone.
-		 *
-		 *   0 = Speech Service enabled
-		 *   1 = Data Service only mode
-		 */
-		g_at_chat_send(nd->chat, "AT_ODO?", none_prefix,
-				NULL, NULL, NULL);
-		g_at_chat_send(nd->chat, "AT_ODO=0", none_prefix,
-				NULL, NULL, NULL);
 		break;
-	case OFONO_VENDOR_STE:
-		g_at_chat_send(nd->chat, "AT+CMER=3,0,0,1", NULL,
+	case OFONO_VENDOR_MBM:
+		g_at_chat_send(nd->chat, "AT*ERINFO=1", none_prefix,
 				NULL, NULL, NULL);
-		g_at_chat_register(nd->chat, "+CIEV:",
-				ste_ciev_notify, FALSE, netreg, NULL);
+		g_at_chat_register(nd->chat, "*ERINFO:", mbm_erinfo_notify,
+					FALSE, netreg, NULL);
+		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
+				cind_support_cb, netreg, NULL);
+		return;
+	case OFONO_VENDOR_NOVATEL:
+		/*
+		 * Novatel doesn't support unsolicited notifications
+		 * of technology changes, but register a handle for
+		 * CNTI so we get notified by any query.
+		 */
+		g_at_chat_register(nd->chat, "$CNTI:", nw_cnti_notify,
+					FALSE, netreg, NULL);
+		break;
+	case OFONO_VENDOR_HUAWEI:
+		g_at_chat_register(nd->chat, "^RSSI:", huawei_rssi_notify,
+					FALSE, netreg, NULL);
 		break;
 	default:
-		break;
+		g_at_chat_send(nd->chat, "AT+CIND=?", cind_prefix,
+				cind_support_cb, netreg, NULL);
+		return;
 	}
 
+	g_at_chat_register(nd->chat, "+CREG:",
+				creg_notify, FALSE, netreg, NULL);
 	ofono_netreg_register(netreg);
 }
 
@@ -794,8 +1038,6 @@ static void at_creg_test_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int creg1 = 0;
 	int creg2 = 0;
 
-	dump_response("creg_read_cb", ok, result);
-
 	if (!ok)
 		goto error;
 
@@ -818,15 +1060,13 @@ static void at_creg_test_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 	if (creg2) {
 		g_at_chat_send(nd->chat, "AT+CREG=2", none_prefix,
-				at_network_registration_initialized,
-				netreg, NULL);
+				at_creg_set_cb, netreg, NULL);
 		return;
 	}
 
 	if (creg1) {
 		g_at_chat_send(nd->chat, "AT+CREG=1", none_prefix,
-				at_network_registration_initialized,
-				netreg, NULL);
+				at_creg_set_cb, netreg, NULL);
 		return;
 	}
 
@@ -845,6 +1085,7 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 
 	nd->chat = chat;
 	nd->vendor = vendor;
+	nd->tech = -1;
 	ofono_netreg_set_data(netreg, nd);
 
 	g_at_chat_send(chat, "AT+CREG=?", creg_prefix,
@@ -866,8 +1107,8 @@ static struct ofono_netreg_driver driver = {
 	.name				= "atmodem",
 	.probe				= at_netreg_probe,
 	.remove				= at_netreg_remove,
-	.registration_status 		= at_registration_status,
-	.current_operator 		= at_current_operator,
+	.registration_status		= at_registration_status,
+	.current_operator		= at_current_operator,
 	.list_operators			= at_list_operators,
 	.register_auto			= at_register_auto,
 	.register_manual		= at_register_manual,
