@@ -128,8 +128,6 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_call *nc, *oc;
 	gboolean poll_again = FALSE;
 
-	dump_response("clcc_poll_cb", ok, result);
-
 	if (!ok) {
 		ofono_error("We are polling CLCC and received an error");
 		ofono_error("All bets are off for call management");
@@ -217,7 +215,6 @@ static void generic_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct voicecall_data *vd = ofono_voicecall_get_data(req->vc);
 	struct ofono_error error;
 
-	dump_response("generic_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (ok && req->affected_types) {
@@ -246,7 +243,6 @@ static void release_id_cb(gboolean ok, GAtResult *result,
 	struct voicecall_data *vd = ofono_voicecall_get_data(req->vc);
 	struct ofono_error error;
 
-	dump_response("release_id_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (ok)
@@ -273,8 +269,6 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_call *call;
 	GSList *l;
 
-	dump_response("atd_cb", ok, result);
-
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok)
@@ -302,7 +296,7 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		else
 			validity = 2;
 
-		ofono_debug("colp_notify: %s %d %d", num, type, validity);
+		DBG("colp_notify: %s %d %d", num, type, validity);
 	}
 
 	/* Generate a voice call that was just dialed, we guess the ID */
@@ -344,9 +338,9 @@ static void at_dial(struct ofono_voicecall *vc,
 	cbd->user = vc;
 
 	if (ph->type == 145)
-		sprintf(buf, "ATD+%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD+%s", ph->number);
 	else
-		sprintf(buf, "ATD%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD%s", ph->number);
 
 	switch (clir) {
 	case OFONO_CLIR_OPTION_INVOCATION:
@@ -476,7 +470,7 @@ static void at_release_specific(struct ofono_voicecall *vc, int id,
 	req->data = data;
 	req->id = id;
 
-	sprintf(buf, "AT+CHLD=1%d", id);
+	snprintf(buf, sizeof(buf), "AT+CHLD=1%d", id);
 
 	if (g_at_chat_send(vd->chat, buf, none_prefix,
 				release_id_cb, req, g_free) > 0)
@@ -494,7 +488,7 @@ static void at_private_chat(struct ofono_voicecall *vc, int id,
 {
 	char buf[32];
 
-	sprintf(buf, "AT+CHLD=2%d", id);
+	snprintf(buf, sizeof(buf), "AT+CHLD=2%d", id);
 	at_template(buf, vc, generic_cb, 0, cb, data);
 }
 
@@ -526,7 +520,7 @@ static void at_deflect(struct ofono_voicecall *vc,
 	char buf[128];
 	unsigned int incoming_or_waiting = (0x1 << 4) | (0x1 << 5);
 
-	sprintf(buf, "AT+CTFR=%s,%d", ph->number, ph->type);
+	snprintf(buf, sizeof(buf), "AT+CTFR=%s,%d", ph->number, ph->type);
 	at_template(buf, vc, generic_cb, incoming_or_waiting, cb, data);
 }
 
@@ -536,7 +530,6 @@ static void vts_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_voicecall_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("vts_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 	cb(&error, cbd->data);
 }
@@ -586,8 +579,6 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct ofono_call *call;
 
-	dump_response("ring_notify", TRUE, result);
-
 	/* See comment in CRING */
 	if (g_slist_find_custom(vd->calls, GINT_TO_POINTER(5),
 				at_util_call_compare_by_status))
@@ -617,9 +608,6 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 	GAtResultIter iter;
 	const char *line;
 	int type;
-	struct ofono_call *call;
-
-	dump_response("cring_notify", TRUE, result);
 
 	/* Handle the following situation:
 	 * Active Call + Waiting Call.  Active Call is Released.  The Waiting
@@ -654,7 +642,7 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 		type = 9;
 
 	/* Generate an incoming call */
-	call = create_call(vc, type, 1, 4, NULL, 128, 2);
+	create_call(vc, type, 1, 4, NULL, 128, 2);
 
 	/* We have a call, and call type but don't know the number and
 	 * must wait for the CLIP to arrive before announcing the call.
@@ -663,7 +651,7 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 	 */
 	vd->clcc_source = g_timeout_add(CLIP_INTERVAL, poll_clcc, vc);
 
-	ofono_debug("cring_notify");
+	DBG("cring_notify");
 }
 
 static void clip_notify(GAtResult *result, gpointer user_data)
@@ -676,8 +664,6 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	GSList *l;
 	struct ofono_call *call;
 
-	dump_response("clip_notify", TRUE, result);
-
 	l = g_slist_find_custom(vd->calls, GINT_TO_POINTER(4),
 				at_util_call_compare_by_status);
 
@@ -710,7 +696,7 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	/* If we have CLI validity field, override our guessed value */
 	g_at_result_iter_next_number(&iter, &validity);
 
-	ofono_debug("clip_notify: %s %d %d", num, type, validity);
+	DBG("clip_notify: %s %d %d", num, type, validity);
 
 	call = l->data;
 
@@ -741,8 +727,6 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 	int num_type, validity, cls;
 	struct ofono_call *call;
 
-	dump_response("ccwa_notify", TRUE, result);
-
 	/* Some modems resend CCWA, ignore it the second time around */
 	if (g_slist_find_custom(vd->calls, GINT_TO_POINTER(5),
 				at_util_call_compare_by_status))
@@ -773,7 +757,7 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 	/* If we have CLI validity field, override our guessed value */
 	g_at_result_iter_next_number(&iter, &validity);
 
-	ofono_debug("ccwa_notify: %s %d %d %d", num, num_type, cls, validity);
+	DBG("ccwa_notify: %s %d %d %d", num, num_type, cls, validity);
 
 	call = create_call(vc, class_to_call_type(cls), 1, 5,
 				num, num_type, validity);
@@ -828,7 +812,7 @@ static void at_voicecall_initialized(gboolean ok, GAtResult *result,
 	struct ofono_voicecall *vc = user_data;
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	ofono_debug("voicecall_init: registering to notifications");
+	DBG("voicecall_init: registering to notifications");
 
 	g_at_chat_register(vd->chat, "RING", ring_notify, FALSE, vc, NULL);
 	g_at_chat_register(vd->chat, "+CRING:", cring_notify, FALSE, vc, NULL);
