@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -39,6 +39,9 @@
 
 #include "hfpmodem.h"
 
+#define POLL_CLCC_INTERVAL 2000
+#define CLIP_TIMEOUT 500
+
 static const char *none_prefix[] = { NULL };
 static const char *clcc_prefix[] = { "+CLCC:", NULL };
 
@@ -50,6 +53,8 @@ struct voicecall_data {
 	unsigned char cind_pos[HFP_INDICATOR_LAST];
 	int cind_val[HFP_INDICATOR_LAST];
 	unsigned int local_release;
+	unsigned int clcc_source;
+	unsigned int clip_source;
 };
 
 struct release_id_req {
@@ -66,6 +71,8 @@ struct change_state_req {
 	int affected_types;
 };
 
+static gboolean poll_clcc(gpointer user_data);
+
 static GSList *find_dialing(GSList *calls)
 {
 	GSList *c;
@@ -197,8 +204,8 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	GSList *calls;
 	GSList *n, *o;
 	struct ofono_call *nc, *oc;
-
-	dump_response("clcc_poll_cb", ok, result);
+	unsigned int num_active = 0;
+	unsigned int num_held = 0;
 
 	if (!ok)
 		return;
@@ -212,6 +219,12 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		nc = n ? n->data : NULL;
 		oc = o ? o->data : NULL;
 
+		if (nc && (nc->status == CALL_STATUS_ACTIVE))
+			num_active++;
+
+		if (nc && (nc->status == CALL_STATUS_HELD))
+			num_held++;
+
 		if (oc && (!nc || (nc->id > oc->id))) {
 			enum ofono_disconnect_reason reason;
 
@@ -243,7 +256,8 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 			 */
 			nc->clip_validity = oc->clip_validity;
 
-			if (memcmp(nc, oc, sizeof(struct ofono_call)) && !nc->type)
+			if (memcmp(nc, oc, sizeof(struct ofono_call)) &&
+					!nc->type)
 				ofono_voicecall_notify(vc, nc);
 
 			n = n->next;
@@ -255,6 +269,27 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	g_slist_free(vd->calls);
 
 	vd->calls = calls;
+
+	/* If either active/held call is more than 1, we are in mpty calls.
+	 * we won't get indicator update if any of them is released by CHLD=1x.
+	 * So we have to poll it.
+	 */
+	if (num_active > 1 || num_held > 1)
+		vd->clcc_source = g_timeout_add(POLL_CLCC_INTERVAL, poll_clcc,
+							vc);
+}
+
+static gboolean poll_clcc(gpointer user_data)
+{
+	struct ofono_voicecall *vc = user_data;
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+
+	g_at_chat_send(vd->chat, "AT+CLCC", clcc_prefix,
+				clcc_poll_cb, vc, NULL);
+
+	vd->clcc_source = 0;
+
+	return FALSE;
 }
 
 static void generic_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -263,7 +298,6 @@ static void generic_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct voicecall_data *vd = ofono_voicecall_get_data(req->vc);
 	struct ofono_error error;
 
-	dump_response("generic_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (ok && req->affected_types) {
@@ -293,8 +327,6 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_call *call;
 	GSList *l;
 
-	dump_response("atd_cb", ok, result);
-
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok)
@@ -337,13 +369,9 @@ static void hfp_dial(struct ofono_voicecall *vc,
 
 	cbd->user = vc;
 	if (ph->type == 145)
-		sprintf(buf, "ATD+%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD+%s", ph->number);
 	else
-		sprintf(buf, "ATD%s", ph->number);
-
-	if ((clir != OFONO_CLIR_OPTION_DEFAULT) ||
-			(cug != OFONO_CUG_OPTION_DEFAULT))
-		goto error;
+		snprintf(buf, sizeof(buf), "ATD%s", ph->number);
 
 	strcat(buf, ";");
 
@@ -393,8 +421,8 @@ static void hfp_answer(struct ofono_voicecall *vc,
 static void hfp_hangup(struct ofono_voicecall *vc,
 			ofono_voicecall_cb_t cb, void *data)
 {
-	/* Hangup all calls */
-	hfp_template("AT+CHUP", vc, generic_cb, 0x3f, cb, data);
+	/* Hangup current active call */
+	hfp_template("AT+CHUP", vc, generic_cb, 0x1, cb, data);
 }
 
 static void hfp_hold_all_active(struct ofono_voicecall *vc,
@@ -417,7 +445,8 @@ static void hfp_release_all_held(struct ofono_voicecall *vc,
 	unsigned int held_status = 0x1 << 1;
 
 	if (vd->ag_mpty_features & AG_CHLD_0) {
-		hfp_template("AT+CHLD=0", vc, generic_cb, held_status, cb, data);
+		hfp_template("AT+CHLD=0", vc, generic_cb, held_status,
+				cb, data);
 		return;
 	}
 
@@ -452,6 +481,104 @@ static void hfp_release_all_active(struct ofono_voicecall *vc,
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
+static void release_id_cb(gboolean ok, GAtResult *result,
+				gpointer user_data)
+{
+	struct release_id_req *req = user_data;
+	struct voicecall_data *vd = ofono_voicecall_get_data(req->vc);
+	struct ofono_error error;
+
+	decode_at_error(&error, g_at_result_final_response(result));
+
+	if (ok)
+		vd->local_release |= (1 << req->id);
+
+	req->cb(&error, req->data);
+}
+
+static void hfp_release_specific(struct ofono_voicecall *vc, int id,
+				ofono_voicecall_cb_t cb, void *data)
+{
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+	struct release_id_req *req = NULL;
+	char buf[32];
+
+	if (!(vd->ag_mpty_features & AG_CHLD_1x))
+		goto error;
+
+	req = g_try_new0(struct release_id_req, 1);
+
+	if (!req)
+		goto error;
+
+	req->vc = vc;
+	req->cb = cb;
+	req->data = data;
+	req->id = id;
+
+	snprintf(buf, sizeof(buf), "AT+CHLD=1%d", id);
+
+	if (g_at_chat_send(vd->chat, buf, none_prefix,
+				release_id_cb, req, g_free) > 0)
+		return;
+
+error:
+	if (req)
+		g_free(req);
+
+	CALLBACK_WITH_FAILURE(cb, data);
+}
+
+static void hfp_private_chat(struct ofono_voicecall *vc, int id,
+				ofono_voicecall_cb_t cb, void *data)
+{
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+	char buf[32];
+
+	if (vd->ag_mpty_features & AG_CHLD_2x) {
+		snprintf(buf, sizeof(buf), "AT+CHLD=2%d", id);
+
+		hfp_template(buf, vc, generic_cb, 0, cb, data);
+
+		return;
+	}
+
+	CALLBACK_WITH_FAILURE(cb, data);
+}
+
+static void hfp_create_multiparty(struct ofono_voicecall *vc,
+					ofono_voicecall_cb_t cb, void *data)
+{
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+
+	if (vd->ag_mpty_features & AG_CHLD_3) {
+		hfp_template("AT+CHLD=3", vc, generic_cb, 0, cb, data);
+
+		return;
+	}
+
+	CALLBACK_WITH_FAILURE(cb, data);
+}
+
+static void hfp_transfer(struct ofono_voicecall *vc,
+			ofono_voicecall_cb_t cb, void *data)
+{
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+	/* Transfer can puts held & active calls together and disconnects
+	 * from both.  However, some networks support transfering of
+	 * dialing/ringing calls as well.
+	 */
+	unsigned int transfer = 0x1 | 0x2 | 0x4 | 0x8;
+
+	if (vd->ag_mpty_features & AG_CHLD_4) {
+		hfp_template("AT+CHLD=4", vc, generic_cb, transfer, cb, data);
+
+		return;
+	}
+
+	CALLBACK_WITH_FAILURE(cb, data);
+}
+
 static void hfp_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 			ofono_voicecall_cb_t cb, void *data)
 {
@@ -505,8 +632,6 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 	int num_type, validity;
 	struct ofono_call *call;
 
-	dump_response("ccwa_notify", TRUE, result);
-
 	/* CCWA can repeat, ignore if we already have an waiting call */
 	if (g_slist_find_custom(vd->calls,
 				GINT_TO_POINTER(CALL_STATUS_WAITING),
@@ -529,18 +654,42 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 	else
 		validity = 2;
 
-	ofono_debug("ccwa_notify: %s %d %d", num, num_type, validity);
+	DBG("ccwa_notify: %s %d %d", num, num_type, validity);
 
 	call = create_call(vc, 0, 1, 5, num, num_type, validity);
 
 	if (!call) {
-		ofono_error("malloc call structfailed. Call management is fubar");
+		ofono_error("malloc call struct failed.  "
+				"Call management is fubar");
 		return;
 	}
 
 	ofono_voicecall_notify(vc, call);
 }
 
+static gboolean clip_timeout(gpointer user_data)
+{
+	struct ofono_voicecall *vc = user_data;
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+	GSList *l;
+	struct ofono_call *call;
+
+	l = g_slist_find_custom(vd->calls,
+				GINT_TO_POINTER(CALL_STATUS_INCOMING),
+				at_util_call_compare_by_status);
+
+	if (l == NULL)
+		return FALSE;
+
+	call = l->data;
+
+	ofono_voicecall_notify(vc, call);
+
+	vd->clip_source = 0;
+
+	return FALSE;
+}
+
 static void ring_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_voicecall *vc = user_data;
@@ -548,8 +697,6 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 	struct ofono_call *call;
 	GSList *waiting;
 
-	dump_response("ring_notify", TRUE, result);
-
 	/* RING can repeat, ignore if we already have an incoming call */
 	if (g_slist_find_custom(vd->calls,
 				GINT_TO_POINTER(CALL_STATUS_INCOMING),
@@ -584,6 +731,11 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 
 	if (!call)
 		ofono_error("Couldn't create call, call management is fubar!");
+
+	/* We don't know the number must wait for CLIP to arrive before
+	 * announcing the call. If timeout, we notify the call as it is.
+	 */
+	vd->clip_source = g_timeout_add(CLIP_TIMEOUT, clip_timeout, vc);
 }
 
 static void clip_notify(GAtResult *result, gpointer user_data)
@@ -596,8 +748,6 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	GSList *l;
 	struct ofono_call *call;
 
-	dump_response("clip_notify", TRUE, result);
-
 	l = g_slist_find_custom(vd->calls,
 				GINT_TO_POINTER(CALL_STATUS_INCOMING),
 				at_util_call_compare_by_status);
@@ -629,7 +779,7 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	g_at_result_iter_skip_next(&iter);
 	g_at_result_iter_skip_next(&iter);
 
-	ofono_debug("clip_notify: %s %d %d", num, type, validity);
+	DBG("clip_notify: %s %d %d", num, type, validity);
 
 	call = l->data;
 
@@ -640,6 +790,11 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	call->clip_validity = validity;
 
 	ofono_voicecall_notify(vc, call);
+
+	if (vd->clip_source) {
+		g_source_remove(vd->clip_source);
+		vd->clip_source = 0;
+	}
 }
 
 static void ciev_call_notify(struct ofono_voicecall *vc,
@@ -709,7 +864,6 @@ static void sync_dialing_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_call *oc;
 	struct ofono_call *nc;
 
-	dump_response("sync_dialing_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok)
@@ -723,7 +877,7 @@ static void sync_dialing_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	/* Look for dialing or alerting calls on the new list */
 	n = find_dialing(calls);
 
- 	/* Let us find if we have done the dial from HF by looking for
+	/* Let us find if we have done the dial from HF by looking for
 	 * existing dialing or alerting calls
 	 */
 	o = find_dialing(vd->calls);
@@ -812,9 +966,10 @@ static void ciev_callsetup_notify(struct ofono_voicecall *vc,
 
 			call->status = CALL_STATUS_ACTIVE;
 			ofono_voicecall_notify(vc, call);
-		} else
+		} else {
 			g_at_chat_send(vd->chat, "AT+CLCC", clcc_prefix,
 					clcc_poll_cb, vc, NULL);
+		}
 
 		break;
 
@@ -839,11 +994,11 @@ static void ciev_callsetup_notify(struct ofono_voicecall *vc,
 
 		if (o) {
 			struct ofono_call *call = o->data;
-			
+
 			call->status = CALL_STATUS_ALERTING;
 			ofono_voicecall_notify(vc, call);
 		}
-	
+
 		break;
 	}
 
@@ -891,10 +1046,10 @@ static void ciev_callheld_notify(struct ofono_voicecall *vc,
 
 				call->status = CALL_STATUS_HELD;
 				ofono_voicecall_notify(vc, call);
-
 			}
-		} else if (callheld == 1)
+		} else if (callheld == 1) {
 			release_with_status(vc, CALL_STATUS_ACTIVE);
+		}
 	}
 
 	vd->cind_val[HFP_INDICATOR_CALLHELD] = value;
@@ -948,7 +1103,7 @@ static void hfp_voicecall_initialized(gboolean ok, GAtResult *result,
 	struct ofono_voicecall *vc = user_data;
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	ofono_debug("hfp_voicecall_init: registering to notifications");
+	DBG("hfp_voicecall_init: registering to notifications");
 
 	g_at_chat_register(vd->chat, "RING", ring_notify, FALSE, vc, NULL);
 	g_at_chat_register(vd->chat, "+CLIP:", clip_notify, FALSE, vc, NULL);
@@ -991,6 +1146,12 @@ static void hfp_voicecall_remove(struct ofono_voicecall *vc)
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
+	if (vd->clcc_source)
+		g_source_remove(vd->clcc_source);
+
+	if (vd->clip_source)
+		g_source_remove(vd->clip_source);
+
 	g_slist_foreach(vd->calls, (GFunc) g_free, NULL);
 	g_slist_free(vd->calls);
 
@@ -1010,10 +1171,10 @@ static struct ofono_voicecall_driver driver = {
 	.release_all_held	= hfp_release_all_held,
 	.set_udub		= hfp_set_udub,
 	.release_all_active	= hfp_release_all_active,
-	.release_specific	= NULL,
-	.private_chat		= NULL,
-	.create_multiparty	= NULL,
-	.transfer		= NULL,
+	.release_specific	= hfp_release_specific,
+	.private_chat		= hfp_private_chat,
+	.create_multiparty	= hfp_create_multiparty,
+	.transfer		= hfp_transfer,
 	.deflect		= NULL,
 	.swap_without_accept	= NULL,
 	.send_tones		= hfp_send_dtmf
