@@ -26,7 +26,6 @@
 #endif
 
 #include <stdint.h>
-#include <stdbool.h>
 #include <glib.h>
 
 #include "client.h"
@@ -38,23 +37,21 @@
 #define COMM_ISI_VERSION_GET_RESP		0x13
 #define COMM_ISA_ENTITY_NOT_REACHABLE_RESP	0x14
 
-#define PN_SIM			0x09
-
 struct verify_data {
 	GIsiVerifyFunc func;
 	void *data;
 };
 
-static bool verify_cb(GIsiClient *client, const void *restrict data,
-			size_t len, uint16_t object, void *opaque)
+static gboolean verify_cb(GIsiClient *client, const void *restrict data,
+				size_t len, uint16_t object, void *opaque)
 {
 	const uint8_t *msg = data;
 	struct verify_data *vd = opaque;
 	GIsiVerifyFunc func = vd->func;
 
-	bool alive = false;
+	gboolean alive = FALSE;
 
-	if(!msg)
+	if (!msg)
 		goto out;
 
 	if (len < 2 || msg[0] != COMMON_MESSAGE)
@@ -62,18 +59,18 @@ static bool verify_cb(GIsiClient *client, const void *restrict data,
 
 	if (msg[1] == COMM_ISI_VERSION_GET_RESP && len >= 4) {
 		g_isi_version_set(client, msg[2], msg[3]);
-		alive = true;
+		alive = TRUE;
 		goto out;
 	}
 
 	if (msg[1] != COMM_ISA_ENTITY_NOT_REACHABLE_RESP)
-		alive = true;
+		alive = TRUE;
 
 out:
 	if (func)
 		func(client, alive, object, vd->data);
 	g_free(vd);
-	return true;
+	return TRUE;
 }
 
 /**
