@@ -330,7 +330,7 @@ static DBusMessage *sim_get_properties(DBusConnection *conn,
 	if (sim->service_numbers && sim->sdn_ready) {
 		service_numbers = get_service_numbers(sim->service_numbers);
 
-		ofono_dbus_dict_append_dict(&dict, "ServiceDiallingNumbers",
+		ofono_dbus_dict_append_dict(&dict, "ServiceNumbers",
 						DBUS_TYPE_STRING,
 						&service_numbers);
 		g_strfreev(service_numbers);
@@ -871,6 +871,10 @@ static void sim_ad_read_cb(int ok, int length, int record,
 
 	new_mnc_length = data[3] & 0xf;
 
+	/* sanity check for potential invalid values */
+	if (new_mnc_length < 2 || new_mnc_length > 3)
+		return;
+
 	if (sim->mnc_length == new_mnc_length)
 		return;
 
@@ -967,7 +971,7 @@ check:
 
 		ofono_dbus_signal_dict_property_changed(conn, path,
 						OFONO_SIM_MANAGER_INTERFACE,
-						"ServiceDiallingNumbers",
+						"ServiceNumbers",
 						DBUS_TYPE_STRING,
 						&service_numbers);
 		g_strfreev(service_numbers);
